<?php
/**
 * Body for CryptoPhoto extension
 * 
 * @file
 * @ingroup Extensions
 */

class CryptoPhoto {
    /**
    * Hook for user preferences
    */
    public static function GetPreferences( $user, &$preferences ) {
        global $wgOut, $wgCryptoPhotoConfig;

        CryptoPhotoDatabaseSupport::loadSettings();

        // test if CryptoPhoto setup is complete
        if ($wgCryptoPhotoConfig['setup_complete']) {
            // CryptoPhoto mandatory flag
            $isMandatory = 0;
            // CryptoPhoto allowed flag
            $isAllowed = 1;
            // CryptoPhoto user option
            $cpEnabled = 0;

            // check if the user is part of the mandatory groups first
            if (count($wgCryptoPhotoConfig['mandatory_groups']) ?
                count(array_intersect($wgCryptoPhotoConfig['mandatory_groups'], $user->getEffectiveGroups())) : 0) {
                // force CryptoPhoto authentication
                $isMandatory = 1;
            } else {
                // check if the user is part of the allowed groups
                if (count($wgCryptoPhotoConfig['groups']) ?
                    count(array_intersect($wgCryptoPhotoConfig['groups'], $user->getEffectiveGroups())) : 0) {
                    // get user option for using CryptoPhoto
                    $cpEnabled = $user->getOption('cryptophoto_enabled', 0);
                } else {
                    // not mandatory or allowed
                    $isAllowed = 0;
                }
            }

            // test if user is allowed to use CryptoPhoto, inform them otherwise
            if (!$isAllowed) {
                $preferences['cp_info'] = array(
                    'section' => 'cryptophoto/cryptophotosettings',
                    'type' => 'info',
                    'default' => '<tr><td colspan="2"><div class="warningbox">' .
                                 wfMessage('cp-not-allowed')->text() .
                                 '</div></td></tr>',
                    'rawrow' => true
                );
                
                return true;
            }

            // test if Cryptophoto usage is required
            if ($isMandatory)
                // inform the user about this
                $preferences['cp_info'] = array(
                    'section' => 'cryptophoto/cryptophotosettings',
                    'type' => 'info',
                    'default' => '<tr><td colspan="2"><div class="warningbox">' .
                                 wfMessage('cp-mandatory')->text() .
                                 '</div></td></tr>',
                    'rawrow' => true
                );
            else 
                // show the CryptoPhoto preference selection
                $preferences['cryptophoto_enabled'] = array(
                    'section' => 'cryptophoto/cryptophotosettings',
                    'type' => 'toggle',
                    'label-message' => 'cryptophoto-pref-enabled',
                );

            $userid = $user->getId();

            // new the cryptophoto instance
            $cp = new CryptoPhotoUtils($wgCryptoPhotoConfig['server'], 
                                       $wgCryptoPhotoConfig['privatekey'], 
                                       $wgCryptoPhotoConfig['publickey'], 
                                       md5($wgCryptoPhotoConfig['salt'] . $userid));

            // start a new session
            $cp->start_session($_SERVER['REMOTE_ADDR']);

            // show the CryptoPhoto token widget
            $preferences['cryptophoto_js'] = array(
                'section' => 'cryptophoto/cryptophotosettings',
                'type' => 'info',
                'label' => '',
                'default' => $cp->get_gen_widget(),
                'raw' => true // if true, the above string won't be html-escaped
            );
        } else {
            // CryptoPhoto setup is not complete. Let the user know
            $preferences['cp_info'] = array(
                'section' => 'cryptophoto/cryptophotosettings',
                'type' => 'info',
                'default' => '<tr><td colspan="2"><div class="warningbox">' .
                             wfMessage('cp-not-setup')->text() .
                             '</div></td></tr>',
                'rawrow' => true
            );
        }
        
        return true;
    }

    /**
    * Hook for Sidebar links (CryptoPhoto Settings ans Users)
    */
    public static function AddSidebarLink($skin, &$bar) {
        global $wgUser, $wgScriptPath;
        
        // Sidebar links only available for administrators
        if (in_array( 'sysop', $wgUser->getEffectiveGroups())) {
            // add CryptoPhoto Settings link
            array_push($bar['navigation'], array(
                'text' => wfMessage('cryptophotosettings')->text(),
                'href' => $wgScriptPath . '/index.php/Special:CryptoPhotoSettings',
                'id' => 'n-cryptophoto-settings',
                'active' => ''
            ));

            // add CryptoPhoto Users link
            array_push($bar['navigation'], array(
                'text' => wfMessage('cryptophotousers')->text(),
                'href' => $wgScriptPath . '/index.php/Special:CryptoPhotoUsers',
                'id' => 'n-cryptophoto-users',
                'active' => ''
            ));
        }
    }
}

?>
