<?php
/**
 * CryptoPhoto database support
 * 
 * @file
 * @ingroup Extensions
 */

class CryptoPhotoDatabaseSupport {
    // CryptoPhoto configuration table
    const CP_CONFIG_TABLE = 'cryptophoto_config'; 

    /**
    * Save CryptoPhoto settings in the database
    */
    public static function saveSettings($publickey, $privatekey, $salt, $server, $mandatory_groups = array(), $groups = array()) {
        // CryptoPhoto confuguration
      	global $wgCryptoPhotoConfig;
      	global $wgDBTableOptions;

      	// new configuration or updating existing one
      	$new = 0;

      	// get database access
      	$dbr = wfGetDB(DB_MASTER);

        // check if CryptoPhoto configuration table exists and if not create it
        if (!$dbr->tableExists(self::CP_CONFIG_TABLE)) {
            // table create query
            $cp_sql = "CREATE TABLE " . self::CP_CONFIG_TABLE . " (\n";
            $cp_sql .= <<<EOT
    keyID INT NOT NULL PRIMARY KEY,
    publickey VARCHAR(35) NOT NULL,
    privatekey VARCHAR(35) NOT NULL,
    salt VARCHAR(35) NOT NULL,
    server VARCHAR(255) NOT NULL,
    mandatory_groups VARCHAR(255) DEFAULT '',
    groups VARCHAR(255) DEFAULT ''
) $wgDBTableOptions;
EOT;

      		// execute query and test result
      		if ($dbr->query($cp_sql) === false) return false;

      		// table just created, we must insert the record
      		$new = 1;
      	} else {
      		// table exists, now check if the record exists
      		$r = $dbr->select(self::CP_CONFIG_TABLE, array('keyID'), 'keyID = 1');

          	// test database error
          	if ($r === false) return false;

          	// record not found (it should be there) so we need to insert it
          	if ($r->numRows() == 0) $new = 1;
      	}

      	// data to be saved
      	$ar = array(
            'publickey' => $publickey,
            'privatekey' => $privatekey,
            'salt' => $salt,
            'server' => $server,
            'mandatory_groups' => implode(',', $mandatory_groups),
            'groups' => implode(',', $groups)
        );

      	// insert/update settings record
      	if ($new) {
      		$ar['keyID'] = 1;
      		if (!$dbr->insert(self::CP_CONFIG_TABLE, $ar)) return false;
      	} else {
      		if (!$dbr->update(self::CP_CONFIG_TABLE, $ar, array('keyID = 1'))) return false;
      	}

      	// mark CryptoPhoto setup as complete
      	$wgCryptoPhotoConfig['setup_complete'] = 1;

      	return true;
    }

    /**
    * Load CryptoPhoto settings from the database
    */
    public static function loadSettings() {
      	global $wgCryptoPhotoConfig;

      	// get database access
      	$dbr = wfGetDB(DB_SLAVE);

      	// nothing to do if the table is not there
      	if (!$dbr->tableExists(self::CP_CONFIG_TABLE)) return false;

      	// get the settings record
      	$r = $dbr->select(self::CP_CONFIG_TABLE, array('publickey', 'privatekey', 'salt', 'server', 'mandatory_groups', 'groups'), 'keyID = 1');

      	// test database error
      	if ($r === false) return false;

      	// make sure the record exists
      	if ($r->numRows() == 0) return false;

      	// fetch the record
      	$settings = $r->fetchRow();

      	// populate our config array
      	$wgCryptoPhotoConfig['publickey'] = $settings['publickey'];
      	$wgCryptoPhotoConfig['privatekey'] = $settings['privatekey'];
      	$wgCryptoPhotoConfig['salt'] = $settings['salt'];
      	$wgCryptoPhotoConfig['server'] = $settings['server'];
      	$wgCryptoPhotoConfig['mandatory_groups'] = explode(',', $settings['mandatory_groups']);
      	$wgCryptoPhotoConfig['groups'] = explode(',', $settings['groups']);
      	$wgCryptoPhotoConfig['setup_complete'] = 1;

      	return true;
    }
}

?>
