<?php
/**
 * CryptoPhoto authentication
 * 
 * @file
 * @ingroup Extensions
 */

class SpecialCryptoPhotoAuth extends SpecialPage {
    function __construct() {
        parent::__construct( 'CryptoPhotoAuth', '', false );
    }

    function execute( $par ) {
        global $wgUser, $mediaWiki, $wgRequest, $wgOut, $wgCryptoPhotoConfig, $IP, $wgServer, $wgScriptPath, $wgSecretKey;

        $this->setHeaders();

        // load CryptoPhoto settings
        CryptoPhotoDatabaseSupport::loadSettings();

        // check for authenticated user
        if ($wgUser->isLoggedIn()) {
            // check if CryptoPhoto setup is complete
            if (!$wgCryptoPhotoConfig['setup_complete']) {
                self::redirect();
                return;
            }

            // check if an admin has overridden the CryptoPhoto authentication
            if ($wgUser->getOption('cryptophoto_admin_override', 0)) {
                // disable CryptoPhoto authentication override for the next login
                $wgUser->setOption('cryptophoto_admin_override', 0);
                $wgUser->saveSettings();

                // redirect the user to the CryptoPhoto Prefrences section
                self::redirect($wgScriptPath . '/index.php/Special:Preferences#mw-prefsection-cryptophoto');
                return;
            }

            // CryptoPhoto authentication disabled by default
            $cpAuth = 0;

            // check if the user is part of the mandatory groups first
            if (count($wgCryptoPhotoConfig['mandatory_groups']) ?
                count(array_intersect($wgCryptoPhotoConfig['mandatory_groups'], $wgUser->getEffectiveGroups())) : 0) {
                // force CryptoPhoto authentication
                $cpAuth = 1;
            } else {
                // check if the user is part of the allowed groups
                if (count($wgCryptoPhotoConfig['groups']) ?
                    count(array_intersect($wgCryptoPhotoConfig['groups'], $wgUser->getEffectiveGroups())) : 0) {
                    // get user option for using CryptoPhoto
                    $cpAuth = $wgUser->getOption('cryptophoto_enabled', 0);
                }
            }

            // check if CryptoPhoto is available for the current user
            if ($cpAuth) {
                $userid = $wgUser->getId();
                
                $cp = new CryptoPhotoUtils($wgCryptoPhotoConfig['server'],
                                           $wgCryptoPhotoConfig['privatekey'],
                                           $wgCryptoPhotoConfig['publickey'], 
                                           md5($wgCryptoPhotoConfig['salt'] . $userid));
        
                $r = $cp->start_session($_SERVER['REMOTE_ADDR'], true);
            
                // check whether the user has previously downloaded a token or not
                if ($cp->has_token != 'true') {
                    //if the user doesn't have a token, he will logged in as ususal
                    self::redirect();
                } else {
                    //for users with a downloaded token, CP authentication is required
                    $wgOut->addHtml("<form method='POST'>".$cp->get_auth_widget()."<input type='submit' name='cp_auth' value='Authenticate'></form>");
                    $wgUser->logout();
                    $_SESSION['cpid'] = $userid;
                }
            } else {
                self::redirect();
                return;
            }
        } elseif ($_SESSION['cpid']) {
            //guest detected

            //new cryptophoto instance
            $cp = new CryptoPhotoUtils($wgCryptoPhotoConfig['server'],
                                       $wgCryptoPhotoConfig['privatekey'], 
                                       $wgCryptoPhotoConfig['publickey'], 
                                       md5($wgCryptoPhotoConfig['salt'] . $_SESSION['cpid']));
      
            // user posted something
            if ((isset($_POST["cp_auth"]) && !empty($_POST["cp_auth"])) || isset($_POST["cp_phc"])) {
                // test for non-empty or blank post
                if ((!empty($_POST["token_response_field_row"]) && !empty($_POST["token_response_field_col"])) || (!empty($_POST["cp_phc"]))) {
                    // check the post
                    if ($_POST["token_selector"] && ( ($_POST["token_response_field_row"] && $_POST["token_response_field_col"]) || $_POST["cp_phc"]) ) {
                        $selector = $_POST["token_selector"];
                        $row_code = $_POST["token_response_field_row"] ? $_POST["token_response_field_row"] : "";
                        $col_code = $_POST["token_response_field_col"] ? $_POST["token_response_field_col"] : "";
                        $cp_phc   = $_POST["cp_phc"] ? $_POST["cp_phc"] : "";
                        $ip       = $_SERVER['REMOTE_ADDR'];
      
                        $cp->verify_response($selector, $row_code, $col_code, $cp_phc, $ip);
            
                        // codes ok, let's finally login the user
                        if ($cp->is_valid) {
                            $wgUser->setId($_SESSION['cpid']);
                            $_SESSION['cpid'] = NULL;
                            unset($_SESSION['cpid']);
                        
                            $wgUser->loadFromId();
                            $wgUser->setCookies();
                            $wgOut->addHtml("<script type='text/javascript'> history.go(1); window.onload=function(){ setTimeout(function(){window.location = 'index.php?title=Main_Page'}, 1000); } </script>" . "<p>" . wfMessage('cp-auth-success')->text() . "</p>");
                        } else {
                            $cperror = $cp->error;
                            $cp->start_session($_SERVER['REMOTE_ADDR'], true);

                            $error_message = wfMessage('cp-auth-error-generic')->text();

                            if ($cperror == "incorrect-retry-count") {
                                $error_message = wfMessage('cp-auth-error-retry-count')->text();
                            }
                        
                            if ($cperror == "invalid-token-response") {
                                $error_message = wfMessage('cp-auth-error-not-match')->text();
                            }

                            $wgOut->addHtml("<form method='POST'><p style='color:#f00'>". $error_message . "</p>" .$cp->get_auth_widget()."<input type='submit' name='cp_auth' value='" . wfMessage('cp-auth-button')->text() . "'></form>"); 
                        }
                    }
                } else {
                    // empty input codes
                    $cp->start_session($_SERVER['REMOTE_ADDR'], true);
                    $wgOut->addHtml("<form method='POST'><p style='color:#f00'>" . wfMessage('cp-auth-error-both-codes')->text() . "</p>".$cp->get_auth_widget()."<input type='submit' name='cp_auth' value='" . wfMessage('cp-auth-button')->text() . "'></form>");
                }
            } else {
                // empty post
                $cp->start_session($_SERVER['REMOTE_ADDR'], true);
                $wgOut->addHtml("<form method='POST'>".$cp->get_auth_widget()."<input type='submit' name='cp_auth' value='" . wfMessage('cp-auth-button')->text() . "'></form>");
            }
        } else {
            //attempt to reach the PC authentication page, without valid login
            $wgOut->addWikiText(wfMessage('cp-auth-error-not-logged')->text());
        }
    }
    
    /**
    * Redirect support
    */
    function redirect ($page = "") {
        global $wgUser, $mediaWiki, $wgRequest, $wgOut;
        if ($page) {
            $wgOut->redirect($page);
            return;
        }

        $returnto = $wgRequest->getVal("returnto");
        if ($returnto) {
            $wgOut->redirect($returnto); //action=purge is used to purge the cache
        } else {
            $wgOut->redirect("index.php?title=Main_Page");
        }
    }
}

?>
